@@ -1,6 +1,6 @@
 /* This entire file is licensed under MIT
  *
- * Copyright 2020 William Wold
+ * Copyright 2020 Sophie Winter
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
  *
@@ -59,13 +59,26 @@ CustomShellSurface *gtk_window_get_custom_shell_surface (GtkWindow *gtk_window);
 
 GtkWindow *custom_shell_surface_get_gtk_window (CustomShellSurface *self);
 
-// In theory this could commit once on next event loop, but for now it will just commit every time it is called
+// Schedules commit on the next frame callback
 // Does nothing is the shell surface does not currently have a GdkWindow with a wl_surface
 void custom_shell_surface_needs_commit (CustomShellSurface *self);
 
+// Attempts to commit the surface immediately
+// Does nothing is the shell surface does not currently have a GdkWindow with a wl_surface
+void custom_shell_surface_force_commit (CustomShellSurface *self);
+
 // Unmap and remap a currently mapped shell surface
 void custom_shell_surface_remap (CustomShellSurface *self);
 
+// Calls virtual->get_popup and adds the surface to the list of popups
+struct xdg_popup* custom_shell_surface_add_popup (CustomShellSurface *self,
+                                                  CustomShellSurface *popup,
+                                                  struct xdg_surface *popup_xdg_surface,
+                                                  struct xdg_positioner *positioner);
+
+// Unamps all popups and then calls virtual->unmap()
+void custom_shell_surface_unmap (CustomShellSurface *self);
+
 // Destruction is taken care of automatically when the associated window is destroyed
 
 #endif // CUSTOM_SHELL_SURFACE_H
