@@ -60,7 +60,7 @@ layer_surface_send_set_size (LayerSurface *self)
  * Sets the window's geometry hints (used to force the window to be a specific size)
  * Needs to be called whenever last_configure_size or anchors are changed
  * Lets windows decide their own size along any axis the surface is not stretched along
- * Forces window (by setting the max and min hints) to be of configured size along axises they are stretched along
+ * Forces window (by setting the max and min hints) to be of configured size along axes they are stretched along
  */
 static void
 layer_surface_update_size (LayerSurface *self)
@@ -208,7 +208,7 @@ static void
 layer_surface_finalize (CustomShellSurface *super)
 {
     LayerSurface *self = (LayerSurface *)super;
-    layer_surface_unmap (super);
+    custom_shell_surface_unmap (super);
     g_free ((gpointer)self->name_space);
 }
 
