@@ -54,20 +54,19 @@ static const struct xdg_surface_listener xdg_surface_listener = {
 static void
 xdg_popup_handle_configure (void *data,
                             struct xdg_popup *_xdg_popup,
-                            int32_t _x,
-                            int32_t _y,
+                            int32_t x,
+                            int32_t y,
                             int32_t width,
                             int32_t height)
 {
     XdgPopupSurface *self = data;
     (void)_xdg_popup;
-    (void)_x;
-    (void)_y;
 
     g_return_if_fail(width >= 0 && height >= 0); // Protocol error
 
     // Technically this should not be applied until we get a xdg_surface.configure
     GtkWindow *gtk_window = custom_shell_surface_get_gtk_window ((CustomShellSurface *)self);
+    gtk_window_move (gtk_window, x, y);
     gtk_window_resize (gtk_window, width, height);
     GdkWindow *gdk_window = gtk_widget_get_window (GTK_WIDGET (gtk_window));
     g_return_if_fail (gdk_window);
@@ -162,7 +161,7 @@ xdg_popup_surface_map (CustomShellSurface *super, struct wl_surface *wl_surface)
     struct xdg_wm_base *xdg_wm_base_global = gtk_wayland_get_xdg_wm_base_global ();
     g_return_if_fail (xdg_wm_base_global);
     struct xdg_positioner *positioner = xdg_wm_base_create_positioner (xdg_wm_base_global);
-    self->geom = gtk_wayland_get_logical_geom (gtk_window);
+    self->geom = gtk_window_get_priv_logical_geom (gtk_window);
     enum xdg_positioner_anchor anchor = gdk_gravity_get_xdg_positioner_anchor(self->position.rect_anchor);
     enum xdg_positioner_gravity gravity = gdk_gravity_get_xdg_positioner_gravity(self->position.window_anchor);
     enum xdg_positioner_constraint_adjustment constraint_adjustment =
@@ -179,9 +178,10 @@ xdg_popup_surface_map (CustomShellSurface *super, struct wl_surface *wl_surface)
     xdg_surface_add_listener (self->xdg_surface, &xdg_surface_listener, self);
 
     CustomShellSurface *transient_for_shell_surface = self->position.transient_for_shell_surface;
-    self->xdg_popup = transient_for_shell_surface->virtual->get_popup (transient_for_shell_surface,
-                                                                       self->xdg_surface,
-                                                                       positioner);
+    self->xdg_popup = custom_shell_surface_add_popup (transient_for_shell_surface,
+                                                      super,
+                                                      self->xdg_surface,
+                                                      positioner);
     g_return_if_fail (self->xdg_popup);
     xdg_popup_add_listener (self->xdg_popup, &xdg_popup_listener, self);
 
@@ -214,7 +214,7 @@ xdg_popup_surface_unmap (CustomShellSurface *super)
 static void
 xdg_popup_surface_finalize (CustomShellSurface *super)
 {
-    xdg_popup_surface_unmap (super);
+    custom_shell_surface_unmap (super);
 }
 
 static struct xdg_popup *
@@ -258,7 +258,7 @@ xdg_popup_surface_on_size_allocate (GtkWidget *_widget,
         self->cached_allocation = *allocation;
         // allocation only used for catching duplicate calls. To get the correct geom we need to check something else
         GtkWindow *gtk_window = custom_shell_surface_get_gtk_window ((CustomShellSurface *)self);
-        self->geom = gtk_wayland_get_logical_geom (gtk_window);
+        self->geom = gtk_window_get_priv_logical_geom (gtk_window);
         xdg_surface_set_window_geometry (self->xdg_surface,
                                          self->geom.x,
                                          self->geom.y,
